@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2006 SuSE Linux Products GmbH Nuernberg, Germany.
  * Copyright (c) 1999, 2000, 2002, 2003 SuSE GmbH Nuernberg, Germany.
  *               2004 SUSE LINUX AG, Germany.
  *               2005 SUSE LINUX Products GmbH, Germany.
@@ -48,10 +49,14 @@
 #include <stdlib.h>
 #include <string.h>
 #include <shadow.h>
+#include <syslog.h>
 #include <sys/types.h>
 
 #define PAM_SM_ACCOUNT
 #include <security/pam_modules.h>
+#if defined (HAVE_SECURITY_PAM_EXT_H)
+#include <security/pam_ext.h>
+#endif
 
 #include "public.h"
 
@@ -72,7 +77,7 @@ hp_expire (pam_handle_t *pamh, int flags, const struct passwd *pw)
   if (max < 0)
     {
     error_state:
-      __pam_log (LOG_ERR, "Age field for %s is wrong", pw->pw_name);
+      pam_syslog (pamh, LOG_ERR, "Age field for %s is wrong", pw->pw_name);
       return PAM_ACCT_EXPIRED;
     }
   ++age;
@@ -119,7 +124,7 @@ expire (pam_handle_t *pamh, int flags, const struct spwd *sp, uid_t uid)
   if (sp->sp_lstchg == 0)
     {
       __write_message (pamh, flags, PAM_TEXT_INFO,
-                       _("Password changing requested. Choose a new password."));
+                       _("Password change requested. Choose a new password."));
       return PAM_NEW_AUTHTOK_REQD;
     }
 
@@ -165,14 +170,14 @@ pam_sm_acct_mgmt (pam_handle_t *pamh, int flags, int argc, const char **argv)
 
   memset (&options, 0, sizeof (options));
 
-  if (get_options (&options, "account", argc, argv) < 0)
+  if (get_options (pamh, &options, "account", argc, argv) < 0)
     {
-      __pam_log (LOG_ERR, "cannot get options");
-      return PAM_BUF_ERR;
+      pam_syslog (pamh, LOG_ERR, "cannot get options");
+      return PAM_SYSTEM_ERR;
     }
 
   if (options.debug)
-    __pam_log (LOG_DEBUG, "pam_sm_acct_mgmt() called");
+    pam_syslog (pamh, LOG_DEBUG, "pam_sm_acct_mgmt() called");
 
   /* get the user name */
   if ((retval = pam_get_user (pamh, &name, NULL)) != PAM_SUCCESS)
@@ -183,15 +188,15 @@ pam_sm_acct_mgmt (pam_handle_t *pamh, int flags, int argc, const char **argv)
     {
       if (name)
 	{
-	  __pam_log (LOG_ERR, "bad username [%s]", name);
+	  pam_syslog (pamh, LOG_ERR, "bad username [%s]", name);
 	  return PAM_USER_UNKNOWN;
 	}
       else if (options.debug)
-	__pam_log (LOG_DEBUG, "name == NULL, return PAM_SERVICE_ERR");
+	pam_syslog (pamh, LOG_DEBUG, "name == NULL, return PAM_SERVICE_ERR");
       return PAM_SERVICE_ERR;
     }
   else if (options.debug)
-    __pam_log (LOG_DEBUG, "username=[%s]", name);
+    pam_syslog (pamh, LOG_DEBUG, "username=[%s]", name);
 
   while (getpwnam_r (name, &pw_resultbuf, pw_buffer, pw_buflen, &pw) != 0
          && errno == ERANGE)
@@ -204,7 +209,7 @@ pam_sm_acct_mgmt (pam_handle_t *pamh, int flags, int argc, const char **argv)
   if (pw == NULL)
     {
       if (options.debug)
-	__pam_log (LOG_DEBUG, "Cannot find passwd entry for %s", name);
+	pam_syslog (pamh, LOG_DEBUG, "Cannot find passwd entry for %s", name);
       return PAM_USER_UNKNOWN;
     }
 
@@ -224,7 +229,9 @@ pam_sm_acct_mgmt (pam_handle_t *pamh, int flags, int argc, const char **argv)
 				&options);
 
 	  if (retval == PAM_SUCCESS ||
-	      retval == PAM_PERM_DENIED || retval == PAM_ACCT_EXPIRED)
+	      retval == PAM_PERM_DENIED ||
+	      retval == PAM_ACCT_EXPIRED ||
+	      retval == PAM_NEW_AUTHTOK_REQD)
 	    return retval;
 	}
     }
@@ -234,9 +241,9 @@ pam_sm_acct_mgmt (pam_handle_t *pamh, int flags, int argc, const char **argv)
       if (options.debug)
 	{
 	  if (pw->pw_passwd == NULL)
-	    __pam_log (LOG_DEBUG, "Password entry is empty for %s", name);
+	    pam_syslog (pamh, LOG_DEBUG, "Password entry is empty for %s", name);
 	  else
-	    __pam_log (LOG_DEBUG, "Account is locked for %s", name);
+	    pam_syslog (pamh, LOG_DEBUG, "Account is locked for %s", name);
 	}
       return PAM_PERM_DENIED;
     }
@@ -255,36 +262,11 @@ pam_sm_acct_mgmt (pam_handle_t *pamh, int flags, int argc, const char **argv)
   if (sp == NULL) /* We have no shadow */
     return PAM_SUCCESS;
 
-  if (sp->sp_pwdp && strcmp (sp->sp_pwdp, "*NP*") == 0)
-    {
-      uid_t save_uid = geteuid ();
-
-      if (seteuid (pw->pw_uid) < 0)
-        {
-          __pam_log (LOG_ERR, "account: seteuid(%d) faild", pw->pw_uid);
-          return PAM_PERM_DENIED;
-        }
-
-      while (getspnam_r (pw->pw_name, &sp_resultbuf, sp_buffer,
-			 sp_buflen, &sp) != 0 && errno == ERANGE)
-	{
-	  errno = 0;
-	  sp_buflen += 256;
-	  sp_buffer = alloca (sp_buflen);
-	}
-
-      if (seteuid (save_uid) < 0)
-        {
-          __pam_log (LOG_ERR, "account: seteuid(%d) faild", save_uid);
-          return PAM_PERM_DENIED;
-        }
-    }
-
   /* Always call expire, could be that root is enforced to change
      root password.  */
   retval = expire (pamh, flags, sp, pw->pw_uid);
   if (options.debug)
-    __pam_log (LOG_DEBUG, "expire() returned with %d", retval);
+    pam_syslog (pamh, LOG_DEBUG, "expire() returned with %d", retval);
   if (retval != PAM_SUCCESS)
     return retval;
 
